/*-
 * Copyright 2003 John-Mark Gurney
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: saa711x.c,v 1.2 2003/04/30 16:25:48 jmg Exp $
 */

/*
 * Cribbed from iicsmb
 */

#include <sys/param.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#include <sys/module.h>
#include <sys/uio.h>

#include <sys/bus.h>
#include "device_if.h"

#include <dev/iicbus/iiconf.h>
#include <dev/iicbus/iicbus.h>
#include "iicbus_if.h"

#include <dev/videocode/ioctl_video.h>
#include <dev/videocode/videocode.h>


#define S7_SOFTC						\
	struct saa711x_softc *s7				\
	    = (struct saa711x_softc *)device_get_softc(dev)

#define SAA7110_SA_A	0x9c	/* if pin 4 is low */
#define SAA7110_SA_B	0x9e	/* if pin 4 is high */
#define SAA7110_MAXINP	8
#define SAA7110_MAXOUT	0
#define SAA7110_STDMODE_REG	0x0D
#define SAA7110_IOCLK_REG	0x0E
#define   SAA7110_IOCLK_OUTEN	0x18
#define   SAA7110_IOCLK_OUTDIS	0x80
#define SAA7110_CTL1_REG	0x0F
#define SAA7110_CTL2_REG	0x10
#define SAA7110_CHRGAIN_REG	0x11

#define IICBUS_TIMEOUT 100     /* us */

struct saa711x_ivars {
	const	char *desc;
	u_char	addr;
	int	maxinp;
	int	maxout;
};

struct saa711x_softc {
	const	struct saa711x_ivars *s7_ivars;
#define s7_slvaddr	s7_ivars->addr
#define s7_maxinp	s7_ivars->maxinp
#define s7_maxout	s7_ivars->maxout

	u_char	s7_rev;

	u_int	s7_inp;
	video_mode_t	s7_mode;
	u_int	s7_outen	: 1;

	device_t s7_iicbus;
	device_t s7_self;
};

#define DGET_BUS(dev)	iicbus_request_bus(dev, dev, IIC_WAIT)
#define GET_BUS(s7)	iicbus_request_bus(s7->s7_iicbus,	\
			    s7->s7_self, IIC_WAIT)
#define DREL_BUS(dev)	iicbus_release_bus(dev, dev)
#define REL_BUS(s7)	iicbus_release_bus(s7->s7_iicbus,	\
			    s7->s7_self)
#define RD_START(s7)	iicbus_start(s7->s7_iicbus,		\
			    s7->s7_slvaddr|1, IICBUS_TIMEOUT)
#define WR_START(s7)	iicbus_start(s7->s7_iicbus,		\
			    s7->s7_slvaddr, IICBUS_TIMEOUT)
#define STOP(s7)	iicbus_stop(s7->s7_iicbus)
#define DSTOP(dev)	iicbus_stop(dev)

static int
s7_write(struct saa711x_softc *s7, u_char reg, u_char byte)
{
	int error = 0;

	if ((error = GET_BUS(s7)))
		return error;
	if ((error = WR_START(s7)))
		goto fail1;
	if ((error = iicbus_write_byte(s7->s7_iicbus, reg, IICBUS_TIMEOUT)))
		goto fail2;
	error = iicbus_write_byte(s7->s7_iicbus, byte, IICBUS_TIMEOUT);
fail2:
	STOP(s7);
fail1:
	REL_BUS(s7);
	return error;
}

static int
s7_read(struct saa711x_softc *s7, u_char reg, u_char *byte)
{
	int error = 0;

	if ((error = GET_BUS(s7)))
		return error;
	if ((error = RD_START(s7)))
		goto fail1;
	error = iicbus_read_byte(s7->s7_iicbus, byte, IICBUS_TIMEOUT);
	STOP(s7);
fail1:
	REL_BUS(s7);
	return error;
}

static const struct tvnorm f50sqpixel = { 944, 768, 83, 880, 625, 576, 16};
static const struct tvnorm f60sqpixel = { 780, 640, 51, 716, 525, 480, 12};

const char saa7110_desc[] = "Philips SAA7110(A) Video Decoder";
const struct saa711x_ivars chips[] = {
	{ saa7110_desc, SAA7110_SA_A, SAA7110_MAXINP, SAA7110_MAXOUT },
	{ saa7110_desc, SAA7110_SA_B, SAA7110_MAXINP, SAA7110_MAXOUT }
};

static const u_char inpmux[][8] = {
	{ 0x06, 0x20, 0x21, 0x22, 0x2c, 0x30, 0x31, 0x21 },
	{ 0x00, 0xd9, 0x17, 0x40, 0x03, 0x44, 0x75, 0x16 },	/* 0 */
	{ 0x00, 0xd8, 0x17, 0x40, 0x03, 0x44, 0x75, 0x16 },	/* 1 */
	{ 0x00, 0xba, 0x07, 0x91, 0x03, 0x60, 0xb5, 0x05 },	/* 2 */
	{ 0x00, 0xb8, 0x07, 0x91, 0x03, 0x60, 0xb5, 0x05 },	/* 3 */
	{ 0x00, 0x7c, 0x07, 0xd2, 0x83, 0x60, 0xb5, 0x03 },	/* 4 */
	{ 0x00, 0x78, 0x07, 0xd2, 0x83, 0x60, 0xb5, 0x03 },	/* 5 */
	{ 0x80, 0x59, 0x17, 0x42, 0xa3, 0x44, 0x75, 0x12 },	/* 6 */
	{ 0x80, 0x9a, 0x17, 0xb1, 0x13, 0x60, 0xb5, 0x14 },	/* 7 */
	{ 0x80, 0x3c, 0x27, 0xc1, 0x23, 0x44, 0x75, 0x21 }	/* 8 */
};

static int
set_inpmux(device_t dev, u_int mux)
{
	u_char buf[2];
	int error, oerror;
	int i, j, llen, lcnt;
	S7_SOFTC;

	llen = sizeof inpmux[0] / sizeof *inpmux[0];
	lcnt = sizeof inpmux / sizeof *inpmux;

	/* prevent looping, only inc after bounds check */
	if (mux >= lcnt - 1) {
		error = EINVAL;
		goto fail;
	}

	if ((error = GET_BUS(s7)))
		return error;

	for (i = 0; !error && i < llen; i += j) {
		buf[0] = inpmux[0][i];
		buf[1] = inpmux[mux + 1][i];
		error = iicbus_block_write(s7->s7_iicbus, s7->s7_slvaddr,
		    buf, 2, &j);
		if (j == 2)
			j = 1;
		else
			j = 0;
	}

	/* Mode should be setup now */
fail:
	oerror = error;
	if ((error = REL_BUS(s7)))
		device_printf(dev, "error releasing iicbus\n");

        return oerror;
}

static const u_char chipinit[] = { 0x00,	/* start address */
    0x4c, 0x3c, 0x0d, 0xef, 0xbd, 0xf0, 0x00, 0x00,
    0xf8, 0xf8, 0x60, 0x60, 0x00, 0x06, 0x18, 0x90,
    0x00, 0x2c, 0x40, 0x46, 0x42, 0x1a, 0xff, 0xda,
    0xf0, 0x8b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0xd9, 0x17, 0x40, 0x41, 0x80, 0x41, 0x80, 0x4f,
    0xfe, 0x01, 0xcf, 0x0f, 0x03, 0x01, 0x81, 0x03,
    0x44, 0x75, 0x01, 0x8c, 0x03 };

static int
find_mode(device_t dev, video_mode_t *new)
{
	int error;
	int i;
	u_char status;
	S7_SOFTC;

	*new = VIDEO_MODE_ERROR;

	if ((error = GET_BUS(s7)))
		return error;

	/* Avoid a -Wcast-qual warning */
	error = iicbus_block_write(s7->s7_iicbus, s7->s7_slvaddr,
	    (u_char *)(uintptr_t)chipinit, sizeof chipinit / sizeof *chipinit,
	    &i);

	if ((error = REL_BUS(s7)))
		device_printf(dev,"error releasing iicbus: %d\n", error);

	set_inpmux(dev, s7->s7_inp);

	/* XXX - not sure on priority */
	tsleep(dev, PRI_MAX_KERN, "vidsettle", hz / 4);
	s7_read(s7, 1, &status);
	if (status & 0x40) { /* no change */
		return 0;
	}
	if ((status & 0x03) == 0) { /* no color */
		s7_write(s7, 0x06, 0x83);
		if (status & 0x20) /* 60hz */
			*new = VIDEO_MODE_NTSC;
		else
			*new = VIDEO_MODE_PAL;
		return 0;
	}

	if (status & 0x20) { /* 60hz */
		s7_write(s7, 0x0d, 0x86);
		s7_write(s7, 0x0f, 0x50);
		s7_write(s7, 0x11, 0x2c);
		*new = VIDEO_MODE_NTSC;
		return 0;
	}

	/* 50hz */
	s7_write(s7, 0x0d, 0x86);
	s7_write(s7, 0x0f, 0x10);
	s7_write(s7, 0x11, 0x59);
	tsleep(dev, PRI_MAX_KERN, "vidsettle", hz / 4);
	s7_read(s7, 1, &status);
	if ((status & 0x03) == 0x01) {
		s7_write(s7, 0x0d, 0x87);
		*new = VIDEO_MODE_SECAM;
		return 0;
	}
	*new = VIDEO_MODE_PAL;
	return 0;
}

static void
saa711x_identify(driver_t *driver, device_t parent)
{
	const struct saa711x_ivars *sai;
	device_t dev;
	int error;
	int i, len;

	if ((error = DGET_BUS(parent)))
		return;

	DSTOP(parent);
	IICBUS_RESET(parent, IIC_FASTEST, 0, NULL);

	/* Probe for chips */
	len = sizeof chips / sizeof *chips;
	for (i = 0; i < len; i++) {
		if ((error = iicbus_start(parent, chips[i].addr,
		    IICBUS_TIMEOUT)))
			continue;

		DSTOP(parent);
		/* we have a chip that responded */
		sai = &chips[i];
		dev = device_add_child(parent, driver->name, -1);
		device_set_ivars(dev, (void *)(uintptr_t)sai);
	}

	IICBUS_RESET(parent, IIC_FASTEST, 0, NULL);
	DREL_BUS(parent);
}

static int
saa711x_probe(device_t dev)
{
	struct saa711x_ivars *sai = device_get_ivars(dev);
	device_t iicbus = device_get_parent(dev);
	S7_SOFTC;
	int error;
#if 0
	int i, len;
	const u_char chips[] = { SAA7110_SA_A, SAA7110_SA_B };
#endif

	if (sai == NULL)
		return ENXIO;

	memset(s7, 0, sizeof *s7);
	s7->s7_ivars = sai;
	s7->s7_iicbus = iicbus;
	s7->s7_self = dev;

#if 0
	if ((error = GET_BUS(s7)))
		return error;

	STOP(s7);
	IICBUS_RESET(iicbus, IIC_FASTEST, 0, NULL);

	/* Probe for chips */
	len = sizeof chips / sizeof *chips;
	for (i = 0; i < len; i++) {
		if ((error = iicbus_start(iicbus, chips[i], IICBUS_TIMEOUT))) {
			device_printf(dev, "failed to start bus\n");
			continue;
		}
		break;
	}

	if (i == len) {
		error = REL_BUS(s7);
		return ENXIO;
	}

	STOP(s7);
	IICBUS_RESET(iicbus, IIC_FASTEST, 0, NULL);

	if ((error = REL_BUS(s7)))
		device_printf(dev, "error releasing iicbus in probe\n");

	s7->s7_slvaddr = chips[i];

#endif
	device_set_desc(dev, sai->desc);

	if ((error = s7_read(s7, 0, &s7->s7_rev))) {
		device_printf(dev, "can't read rev: %d\n", error);
		s7->s7_rev = 0;
	}

	return 0;
}

static int
saa711x_attach(device_t dev)
{
	S7_SOFTC;
	device_printf(dev, "attach slave: %02x, rev: %d\n",
	    (int)s7->s7_slvaddr, (int) s7->s7_rev);

	s7->s7_inp = 0;

	find_mode(dev, &s7->s7_mode);

	return 0;
}

static void
saa711x_enable_output(device_t dev)
{
	S7_SOFTC;

	if (!s7->s7_outen)
		s7_write(s7, SAA7110_IOCLK_REG, SAA7110_IOCLK_OUTEN);
	s7->s7_outen = 1;
}

static void
saa711x_disable_output(device_t dev)
{
	S7_SOFTC;

	if (s7->s7_outen)
		s7_write(s7, SAA7110_IOCLK_REG, SAA7110_IOCLK_OUTDIS);
	s7->s7_outen = 0;
}

static int
saa711x_detach(device_t dev)
{
	S7_SOFTC;

	s7_write(s7, 0x0d, 0x7d);
	s7_write(s7, 0x0e, 0x00);
	s7_write(s7, 0x31, 0x00);

	device_set_ivars(dev, NULL);

	return 0;
}

static void
saa711x_get_cap(device_t dev, int *flags, int *inps, int *outs)
{
	S7_SOFTC;

	*flags = VIDEO_PAL | VIDEO_NTSC | VIDEO_SECAM
	    | VIDEO_AUTO;
	*inps = s7->s7_maxinp;
	*outs = s7->s7_maxout;
}

static const struct tvnorm *
saa711x_get_tvnorm(device_t dev)
{
	S7_SOFTC;

	switch (s7->s7_mode) {
	case VIDEO_MODE_PAL:
	case VIDEO_MODE_SECAM:
		return &f50sqpixel;

	case VIDEO_MODE_NTSC:
		return &f60sqpixel;

	case VIDEO_MODE_ERROR:
	default:
		return NULL;
	}
}

static video_mode_t
saa711x_set_mode(device_t dev, video_mode_t mode)
{
	S7_SOFTC;

	switch (mode) {
	case VIDEO_MODE_NTSC:
		s7_write(s7, SAA7110_STDMODE_REG, 0x86);
		s7_write(s7, SAA7110_CTL1_REG, 0x50);
		s7_write(s7, SAA7110_CHRGAIN_REG, 0x2C);
		break;
	case VIDEO_MODE_PAL:
		s7_write(s7, SAA7110_STDMODE_REG, 0x86);
		s7_write(s7, SAA7110_CTL1_REG, 0x10);
		s7_write(s7, SAA7110_CHRGAIN_REG, 0x59);
		break;
	case VIDEO_MODE_SECAM:
		s7_write(s7, SAA7110_STDMODE_REG, 0x87);
		s7_write(s7, SAA7110_CTL1_REG, 0x10);
		s7_write(s7, SAA7110_CHRGAIN_REG, 0x59);
		break;
	case VIDEO_MODE_AUTO:
	default:
		find_mode(dev, &mode);
	}

	s7->s7_mode = mode;
	return mode;
}

static video_mode_t
saa711x_get_mode(device_t dev)
{
	S7_SOFTC;

	return s7->s7_mode;
}

static int
saa711x_set_inp(device_t dev, int inp)
{
	S7_SOFTC;

	if (inp > s7->s7_maxinp || inp < 0)
		return EINVAL;

	s7->s7_inp = inp;
	return set_inpmux(dev, s7->s7_inp);
}

static devclass_t saa711x_devclass;

static device_method_t saa711x_methods[] = {
	/* device interface */
	DEVMETHOD(device_identify,	saa711x_identify),
	DEVMETHOD(device_probe,		saa711x_probe),
	DEVMETHOD(device_attach,	saa711x_attach),
	DEVMETHOD(device_detach,	saa711x_detach),

	/* videocode interface */
	DEVMETHOD(videocode_get_cap,		saa711x_get_cap),
	DEVMETHOD(videocode_get_tvnorm,		saa711x_get_tvnorm),
	DEVMETHOD(videocode_set_mode,		saa711x_set_mode),
	DEVMETHOD(videocode_get_mode,		saa711x_get_mode),
	DEVMETHOD(videocode_set_inp,		saa711x_set_inp),
	/*DEVMETHOD(videocode_set_picture,	saa711x_set_picture),*/
	DEVMETHOD(videocode_enable_output,	saa711x_enable_output),
	DEVMETHOD(videocode_disable_output,	saa711x_disable_output),
	DEVMETHOD(videocode_istype,		videocode_isdecoder),

#if 0
	/* I don't think I need this */
	/* iicbus interface */
        DEVMETHOD(iicbus_intr,          iicsmb_intr),
#endif

	{ 0, 0}
};

static driver_t saa711x_driver = {
        "saa711x",
        saa711x_methods,
	sizeof(struct saa711x_softc)
};

DRIVER_MODULE(saa711x, iicbus, saa711x_driver, saa711x_devclass, 0, 0);
MODULE_DEPEND(saa711x, iicbus, IICBUS_MINVER, IICBUS_PREFVER, IICBUS_MAXVER);
MODULE_DEPEND(saa711x, videocode, VIDEOCODE_MINVER, VIDEOCODE_PREFVER,
	VIDEOCODE_MAXVER);
MODULE_VERSION(saa711x, 1);
