/*-
 * Copyright 2003 John-Mark Gurney
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ioctl_video.h,v 1.6 2003/04/30 16:28:48 jmg Exp jmg $
 */

#ifndef	_IOCTL_VIDEO_H_
#define	_IOCTL_VIDEO_H_	1

/*
 * notinuse define is used to denote parts that are proposed, but have not
 * been actually used by a driver/application yet.
 */

typedef enum video_mode {
#define VIDEO_MODE_MIN	0
	VIDEO_MODE_ERROR=0,
	VIDEO_MODE_AUTO=0,
	VIDEO_MODE_NTSC,
	VIDEO_MODE_PAL,
	VIDEO_MODE_SECAM
#define VIDEO_MODE_MAX	3
} video_mode_t;

#define video_mode_to_str(x) ((x) == VIDEO_MODE_NTSC ? "NTSC"		\
				: (x) == VIDEO_MODE_PAL ? "PAL"		\
				: (x) == VIDEO_MODE_SECAM ? "SECAM"	\
				: (x) == VIDEO_MODE_ERROR ? "error"	\
				: "unknown")

/* structure for DECODER_[GS]ETGEO - get/set geometry  */
struct decoder_geomet {
     u_short	rows;
     u_short	columns;
     u_short	frames;
     u_long	oformat;
};

/*
 * structure for getting and setting direct transfers to video-ram
 *
 * This interface is bad.  Addr must be specified as physical address.
 * Prevents driver from being accessable to a normal user.
 */
struct decoder_vram {
     u_long	addr;		/* Address of location to dma to */
     u_long	width;		/* Width of memory area */
     u_long	banksize;	/* Size of Vram bank */
     u_long	ramsize;	/* Size of Vram */
};

/*
 * structure for getting and setting direct transfers to user space buffer
 * user space buffer must be physically contigious (i.e. mmap /dev/mem w/
 * physical address space of video buffer)
 */
struct decoder_buffer {
	caddr_t	addr;		/* Address of buffer (in user process) */
	int	len;		/* length of memory (in user process) */
	u_int	format;		/* GEO format */
	int	vidwidth;	/* pixels per line */
	int 	xstart;
	int	ystart;
	int	width;		/* window width */
	int	height;		/* window height */
};

struct jpeg_marker {
	u_int8_t	jm_id;	/* E[0-F] for APP0-APPF or FE for COM */
	u_int8_t	jm_len;
	u_char	jm_data[254];
};

struct overlay_mask {
	u_int16_t	om_allvis	: 1;
	u_int16_t	om_allhide	: 1;
	u_int16_t	om_hide		: 1;
	u_int16_t	om_vis		: 1;
	u_int16_t	om_dealloc	: 1;
	u_int16_t	om_cnt;
	struct overlay_box {
		u_int16_t x1, y1;
		u_int16_t x2, y2;
#define VIDEO_OVERLAY_NBOXES	63
	} om_boxes[VIDEO_OVERLAY_NBOXES];
};

/*
 * From ioctl_meteor.h (but with different names)
 */
#define DECODER_CAPTURE	_IOW('x', 1, int)		/* capture a frame */
#define DECODER_CAPTUR	DECODER_CAPTURE
#define DECODER_SETGEO	_IOW('x', 3, struct decoder_geomet)	/* set geometry */
#define DECODER_GETGEO	_IOR('x', 4, struct decoder_geomet)	/* get geometry */
#define DECODER_SET_NORM	_IOW('x', 7, enum video_mode)	/* set norm ntsc/pal/secam */
#define DECODER_GET_NORM	_IOR('x', 7, enum video_mode)	/* get norm */
#define DECODER_SET_INPUT	_IOW('x', 8, unsigned long)	/* set input dev */
#define DECODER_GET_INPUT	_IOR('x', 8, unsigned long)	/* get input dev */
#define	DECODER_SVIDEO	_IOW('x', 13, struct decoder_vram)	/* set video */
#define	DECODER_GVIDEO	_IOR('x', 13, struct decoder_vram)	/* get video */
#define	DECODER_SVIDBUF	_IOW('x', 23, struct decoder_buffer)	/* set vidbuf */
#define	DECODER_GVIDBUF	_IOR('x', 23, struct decoder_buffer)	/* get vidbuf */
#define DECODER_SET_TVNORM	_IOW('x', 24, struct tvnorm)	/* set tvnorm */
#define DECODER_GET_TVNORM	_IOR('x', 24, struct tvnorm)	/* get tvnorm */

					/* VIDEOCAPTUR capture options */
#define DECODER_CAP_CONTINOUS	0x0002	/* continuously capture */
#define DECODER_CAP_STOP_CONT	0x0004	/* stop the continuous capture */

					/* valid video output formats:  */
#define DECODER_GEO_RGB15	0x0004000	/* packed */
#define DECODER_GEO_RGB32	0x0008000	/* unpacked 24bit RGB */
#define DECODER_GEO_RGB16	0x0010000	/* packed -- initialized default */
#define DECODER_GEO_RGB24	0x0020000	/* RBG 24 bits packed */
#define DECODER_GEO_YUV_422	0x4000000	/* 4-2-2 YUV in YUV combined */
#define DECODER_GEO_OUTPUT_MASK	0x40fc000

#define BIT(x)	(1 << x)

/* Bits shared with STATUS */
/* used for get_cap videocode interface call */
#define VIDEO_PAL	BIT(0)
#define VIDEO_NTSC	BIT(1)
#define VIDEO_SECAM	BIT(2)
#define VIDEO_AUTO	BIT(3)

#define MJPEG_SET_APP	_IOW('M', 1, struct jpeg_marker)	/* set app */
#define MJPEG_GET_APP	_IOR('M', 1, struct jpeg_marker)	/* get app */
#define MJPEG_SET_COM	_IOW('M', 2, struct jpeg_marker)	/* set com */
#define MJPEG_GET_COM	_IOR('M', 2, struct jpeg_marker)	/* get com */
#define MJPEG_SET_TVNORM	_IOW('M', 3, struct tvnorm)	/* set tvnorm */
#define MJPEG_GET_TVNORM	_IOR('M', 3, struct tvnorm)	/* get tvnorm */

#define VIDEO_OVERLAY	_IOW('V', 1, struct overlay_mask)	/* overlay */

#ifdef notinuse
#define VIDEO_STATUS_GOOD	BIT(4)
#define VIDEO_STATUS_COLOR	BIT(5)

struct video_caps {
     int	flags;
     int	inputs;
     int	outputs;
};

#define ZR_DC10		1
#define ZR_DC10plus	2
#define ZR_LML33	3
#define ZR_BUZ		4

/*
 | (ED: DECODER is a technical name for video input)
 */
struct decoder_capframe {
     short	command;	/* see below for valid DECODER_CAPFRM commands */
     short	lowat;		/* start transfer if < this number */
     short	hiwat;		/* stop transfer if > this number */
};

/*
 * structure for DECODER_GCOUNT-get count of frames, fifo errors and dma errors
 */
struct decoder_counts {
     u_long fifo_errors;	/* count of fifo errors since open */
     u_long dma_errors;		/* count of dma errors since open */
     u_long frames_captured;	/* count of frames captured since open */
     u_long even_fields_captured; /* count of even fields captured */
     u_long odd_fields_captured; /* count of odd fields captured */
} ;

/*
 * From ioctl_meteor.h (but with different names)
 */
#define DECODER_CAPFRM _IOW('x', 2, struct decoder_capframe)  /* sync capture */
#define DECODER_STATUS _IOR('x', 5, unsigned int)	/* get status */
#define DECODER_SHUE   _IOW('x', 6, signed char)		/* set hue */
#define DECODER_GHUE   _IOR('x', 6, signed char)		/* get hue */

#define	DECODER_SCHCV  _IOW('x', 9, unsigned char)	/* set uv gain */
#define	DECODER_GCHCV  _IOR('x', 9, unsigned char)	/* get uv gain */
#define	DECODER_SCOUNT _IOW('x',10, struct decoder_counts)
#define	DECODER_GCOUNT _IOR('x',10, struct decoder_counts)
#define DECODER_SFPS   _IOW('x',11, unsigned short)	/* set fps */
#define DECODER_GFPS   _IOR('x',11, unsigned short)	/* get fps */
#define DECODER_SSIGNAL _IOW('x', 12, unsigned int)	/* set signal */
#define DECODER_GSIGNAL _IOR('x', 12, unsigned int)	/* get signal */
#define	DECODER_SBRIG  _IOW('x', 14, unsigned char)	/* set brightness */
#define DECODER_GBRIG  _IOR('x', 14, unsigned char)	/* get brightness */
#define	DECODER_SCSAT  _IOW('x', 15, unsigned char)	/* set chroma sat */
#define DECODER_GCSAT  _IOR('x', 15, unsigned char)	/* get uv saturation */
#define	DECODER_SCONT  _IOW('x', 16, unsigned char)	/* set contrast */
#define	DECODER_GCONT  _IOR('x', 16, unsigned char)	/* get contrast */

#define DECODER_SBT254 _IOW('x', 17, unsigned short)	/* set Bt254 reg */
#define DECODER_GBT254 _IOR('x', 17, unsigned short)	/* get Bt254 reg */

#define DECODER_SHWS   _IOW('x', 18, unsigned char)	/* set hor start reg */
#define DECODER_GHWS   _IOR('x', 18, unsigned char)	/* get hor start reg */
#define DECODER_SVWS   _IOW('x', 19, unsigned char)	/* set vert start reg */
#define DECODER_GVWS   _IOR('x', 19, unsigned char)	/* get vert start reg */
#define	DECODER_STS    _IOW('x', 20, unsigned char)	/* set time stamp */
#define	DECODER_GTS    _IOR('x', 20, unsigned char)	/* get time stamp */

/*
 * Custom
 */
#define	DECODER_CAPABILITIES	_IOR('x', 21, struct video_caps)	/* get decoder caps*/
#define	DECODER_SET_PICTURE	_IOW('x', 22, struct video_picture)

/*
 * From Bt848
 */
#define	TVTUNER_SETCHNL    _IOW('x', 32, unsigned int)	/* set channel */
#define	TVTUNER_GETCHNL    _IOR('x', 32, unsigned int)	/* get channel */
#define	TVTUNER_SETTYPE    _IOW('x', 33, unsigned int)	/* set tuner type */
#define	TVTUNER_GETTYPE    _IOR('x', 33, unsigned int)	/* get tuner type */
#define	TVTUNER_GETSTATUS  _IOR('x', 34, unsigned int)	/* get tuner status */
#define	TVTUNER_SETFREQ    _IOW('x', 35, unsigned int)	/* set frequency */
#define	TVTUNER_GETFREQ    _IOR('x', 36, unsigned int)	/* get frequency */


/* clipping */

/*
 *  clipping works like the X11 regions,
 *  passed region contains y-x-banded very constrained boxes
 *  for _visible_ areas. In a way, this is nice; if, no, when the array
 *  fills up, the incomplete list can still be used, only some areas
 *  on the bottom-right "stay still" until window topology changes again.
 */
struct video_region {
     u_short flags;
#define VIDEO_CLIPBOXES_HIDE 0x0001	/* inverted sense */
     short y1, x1, y2, x2;		/* minimal area containing all boxes */
     short nbox;
#define VIDEO_CLIPBOXES_MAX 64
     struct video_region_box {
	  short y1, x1, y2, x2;		/* individual boxes, 1 inclusive, 2 exclusive */
     } boxes[VIDEO_CLIPBOXES_MAX];
};
#define VIDCAP_SETREGION      _IOW('x', 80, struct video_region)
#define VIDCAP_GETREGION      _IOR('x', 80, struct video_region)

#define DECODER_STATUS_STTC	BIT(2)	/* signal is from vcr */

					/* VIDEOCAPTUR capture options */
#define DECODER_CAP_SINGLE	0x0001	/* capture one frame */
#define DECODER_CAP_SINGLE_ASYNC 3 	/* like SINGLE, but don't wait in kernel */

					/* VIDEOCAPFRM capture commands */
#define DECODER_CAP_N_FRAMES	0x0001	/* capture N frames */
#define DECODER_CAP_STOP_FRAMES	0x0002	/* stop capture N frames */
#define	DECODER_HALT_N_FRAMES	0x0003	/* halt of capture N frames */
#define DECODER_CONT_N_FRAMES	0x0004	/* continue after above halt */

#define DECODER_INPUT_DEV0	0x01000	/* camera input 0 -- default */
#define DECODER_INPUT_DEV_RCA	DECODER_INPUT_DEV0
#define DECODER_INPUT_DEV1	0x02000	/* camera input 1 */
#define DECODER_INPUT_DEV2	0x04000	/* camera input 2 */
#define DECODER_INPUT_DEV3	0x08000	/* camera input 3 */
#define DECODER_INPUT_DEV_RGB	0x0a000	/* for rgb version of video */
#define DECODER_INPUT_DEV_SVIDEO	0x06000 /* S-video input port */

					/* valid video output formats:  */
					/* internally stored in 32 bits */
#define DECODER_GEO_YUV_PACKED	0x0040000	/* 4-2-2 YUV 16 bits packed */
#define DECODER_GEO_YUV_PLANAR	0x0080000	/* 4-2-2 YUV 16 bits planer */
#define DECODER_GEO_YUV_PLANER	DECODER_GEO_YUV_PLANAR
#define DECODER_GEO_UNSIGNED	0x0400000	/* unsigned uv outputs */
#define DECODER_GEO_EVEN_ONLY	0x1000000	/* set for even only field capture */
#define DECODER_GEO_ODD_ONLY	0x2000000	/* set for odd only field capture */
#define DECODER_GEO_FIELD_MASK	0x3000000
#define DECODER_GEO_YUV_12	0x10000000	/* YUV 12 format */
#define DECODER_GEO_YUV_9	0x40000000	/* YUV 9 format */

#define	DECODER_FIELD_MODE	0x80000000	/* Field cap or Frame cap */

#define	DECODER_SIG_MODE_MASK	0xffff0000
#define	DECODER_SIG_FRAME		0x00000000	/* signal every frame */
#define	DECODER_SIG_FIELD		0x00010000	/* signal every field */

/* following structure is used to coordinate the synchronous */
	   
struct decoder_mem {
     /* kernel write only  */
     int	frame_size;	 /* row*columns*depth */
     unsigned	num_bufs;	 /* number of frames in buffer (1-32) */
     /* user and kernel change these */
     int	lowat;		 /* kernel starts capture if < this number */
     int	hiwat;		 /* kernel stops capture if > this number.
				    hiwat <= numbufs */
     unsigned	active;	 /* bit mask of active frame buffers
			    kernel sets, user clears */
     int	num_active_bufs; /* count of active frame buffer
				    kernel increments, user decrements */

     /* reference to mmapped data */
     caddr_t	buf;		 /* The real space (virtual addr) */
};

#endif /* notinuse */

#endif /* _IOCTL_VIDEO_H_ */
