#
# Copyright 2003 John-Mark Gurney
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: videocode_if.m,v 1.2 2003/04/30 16:28:48 jmg Exp jmg $
#

#include <sys/bus.h>
#include <dev/videocode/ioctl_video.h>
#include <dev/videocode/videocode.h>

INTERFACE videocode;

#
# Default implementations of some methods.
#
CODE {
	static int isnotsupported(device_t dev)
	{
		return ENXIO;
	}

	static int null_invalid(device_t dev)
	{
		return EINVAL;
	}

	static int null_inout(device_t dev, int i)
	{
	    return EINVAL;
	}
};

# Get capabilities
METHOD void get_cap {
	device_t dev;
	int *flags;
	int *inps;
	int *outs;
};

# Get Parameters for the chip
METHOD const struct tvnorm * get_tvnorm {
	device_t dev;
};

# Status
# 	VIDEO_STATUS_GOOD
# 	VIDEO_STATUS_COLOR
# 	VIDEO_NTSC
# 	VIDEO_PAL
# 	VIDEO_SECAM
METHOD void get_status {
	device_t dev;
	int res;
};

# Set Mode
# 	VIDEO_MODE_ERROR
# 	VIDEO_MODE_NTSC
# 	VIDEO_MODE_PAL
# 	VIDEO_MODE_SECAM
# 	VIDEO_MODE_AUTO
METHOD video_mode_t set_mode {
	device_t dev;
	video_mode_t mode;
};

METHOD video_mode_t get_mode {
	device_t dev;
};

# Set input
METHOD int set_inp {
	device_t dev;
	int inp;
} DEFAULT null_inout;

# Set output
METHOD int set_out {
	device_t dev;
	int out;
} DEFAULT null_inout;

# Enable Output
METHOD void enable_output {
	device_t dev;
} DEFAULT null_invalid;

# Disable Output
METHOD void disable_output {
	device_t dev;
} DEFAULT null_invalid;

# Enable Input
METHOD void enable_input {
	device_t dev;
} DEFAULT null_invalid;

# Disable Input
METHOD void disable_input {
	device_t dev;
} DEFAULT null_invalid;

# Set Picutre
METHOD int set_picture {
	device_t dev;
	int bright;
	int contrast;
	int sat;
	int hue;
};

#Return ENXIO if not supported
# or VIDEO_ENCODER or VIDEO_DECODER
METHOD int istype {
	device_t dev;
} DEFAULT isnotsupported;
