/*-
 * Copyright 2003 John-Mark Gurney
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: zr_reg.h,v 1.16 2003/04/30 16:22:47 jmg Exp $
 */

#ifndef _ZR_REG_H_
#define _ZR_REG_H_ 1

#define ZRSC_PRINTF(sc, ...)	device_printf((sc)->zr_self, __VA_ARGS__)

#include <dev/zr/zr_i2c.h>
#include <dev/zr/zr_os.h>

MALLOC_DECLARE(M_ZR);

#define PCI_VENDOR_ZORAN 0x11de
#define PCI_PRODUCT_ZR360x7 0x6057

/* note that bits 8-15 are used by the system */
#define UNIT(t)		(t)
#define SUBUNIT(t)	(0)
#define ZR_SOFTC	struct zr_softc *zr = device_get_softc(dev)
#define ZR_DEV_SOFTC	struct zr_softc *zr = devclass_get_softc(zr_devclass, \
							UNIT(minor(devt)))

enum zr_state {
	ZR_DISABLED,
	ZR_ENABLED,
	ZR_DEVOPENED,
	ZR_DEVCLOSED,
	ZR_CAPBUF,
	ZR_CAPBUFFREE,
	ZR_CAPTURE_START,
	ZR_CAPTURE_STOP,
	ZR_BUFSET,
	ZR_WINSET,
	ZR_MASKALLOC,
	ZR_MASK,
	ZR_MASKFREE,
	ZR_MASKDEALLOC,
};
typedef enum zr_state zr_state_t;

struct vendor_info {
	const char *desc;
	u_int32_t id;
};

struct zr_subchip {
	char	*desc;
	int	subid;
	int	irq;
	int	reset;
	int	sleep;
};

struct zr_carddesc {
	const char *desc;
	const struct vendor_info *vi;
	u_int32_t	sysid;
	u_int8_t	revid;
	int	irq_vsync;
	/* reduce these to 3 bit field? only 8 gpio pins */
	int	gpio_dir;		/* for GPIO direction setup */
	int	gpio_setframe;
	int	gpio_setvidbusdir;
	int	gpio_setvidbusen;
	int	gpio_freq;
	int	HSPol;
	int	VSPol;
	const char **indesc;
	int	inchannels;
	const int *inchans;
	const char **outdesc;
	int	outchannels;
	const int *outchans;
#define ZR_FORCARDSUBCHIPS(zr, i)					\
		for ((i) = 0; (i) < sizeof zr->zr_card->subchips /	\
		    sizeof *zr->zr_card->subchips &&			\
		    zr->zr_card->subchips[(i)].desc != NULL; (i)++)
	const struct zr_subchip subchips[POSTOFFICE_NUMSUBIDS];
	/*
	 * we should probably add other chips to this table, but then it
	 * looses it's genericness.
	 */
};

struct zr_dmamem {
#ifdef ZR_USE_BUS_DMA
	bus_dma_tag_t	zd_tag;
	bus_dmamap_t	zd_map;
	bus_size_t	zd_size;
	int	zd_maxsegs;
	int	zd_nsegs;
	bus_dma_segment_t zd_segs[1];
#else
	size_t	zd_size;
#endif /* ZR_USE_BUS_DMA */
	vm_offset_t	zd_vaddr;
};

#define ZR_FRAMEBUF	4	/* the ZR36067 only supports four active bufs */

struct zr_softc {
	/* self info */
	device_t	zr_self;
	const char *	zr_xname;

	/* state flags/data */
	struct mtx	zr_lock;
	zr_state_t	zr_state;
	u_int	zr_enabled	: 1;
	u_int	zr_opened	: 1;
	u_int	zr_capture	: 1;
	u_int	zr_bufset	: 1;
	u_int	zr_winset	: 1;
	u_int	zr_capbuf	: 1;
	u_int	zr_nbufs	: 1;
	u_int	zr_maskalloc	: 1;
	u_int	zr_mask		: 1;
	u_int	zr_maskvalid	: 1;
	u_int	zr_defvis	: 1;
	u_int	zr_masklen;
	u_int	zr_maskstride;
	struct {
		int	JPEGRepIRQ;
		int	CodRepIRQ;
		int	GIRQ0;
		int	GIRQ1;
	} zr_irqcnt;

	/* config data */
	os_proc_t	zr_proc;
	struct decoder_buffer zr_buffer;
	u_int32_t	zr_physaddr;

	/* card info */
	unsigned short zr_subvenid;
	unsigned short zr_subsysid;
	u_int8_t	zr_revid;
	const char *	zr_vendor;
	const struct zr_carddesc *zr_card;
	struct po_devinfo *zr_subchips[POSTOFFICE_NUMSUBIDS];
#define ZR_FORSUBCHIPS(zr, i)	for ((i) = 0; (i) < sizeof (zr)->zr_subchips / sizeof *(zr)->zr_subchips && (zr)->zr_subchips[i] != NULL; (i)++)
#define ZR_CALLSUBCHIPS(zr, fun, errvar, expr, ...) 			\
		do {							\
			int i;						\
			struct po_devinfo *pd;				\
			ZR_FORSUBCHIPS((zr), i) {			\
				pd = (zr)->zr_subchips[i];		\
				(errvar) = fun(pd->pd_dev, ##__VA_ARGS__); \
				if ((expr))				\
					break;				\
			}						\
		} while (0)

	/* set once parameters */
	/*u_char	zr_gpio_dir;	 GPIO Direction (from zr_card) */
	u_int8_t	zr_gb_dur[8];	/* Guest Bus Duration timing */
	u_int8_t	zr_gb_rec[8];	/* Guest Bus Recovery timing */

	/* i2c */
	struct zr_i2c_softc zr_i2c_sc;

	/* decoder/encoder */
#define IFCODERINVAL(zr, code)	if (zr_find_coders(zr), zr->zr_##code == NULL)
	device_t	zr_decoder;
	video_mode_t	zr_dec_mode;
	int	zr_dec_inp;
	int	zr_dec_norm_allocated;
	const struct tvnorm *zr_dec_norm;
	device_t	zr_encoder;

	/* device bus data */
	struct {
		/* memory mapping */
		int	mem_rid;
		struct	resource *res_mem;
		bus_space_tag_t     memt;
		bus_space_handle_t  memh;

		/* irq */
		int	irq_rid;
		struct	resource *res_irq;
		void	*res_ih;

		/* bus dma */
		bus_dma_tag_t	zb_tag;
		struct	zr_dmamem zb_mask;
		struct	zr_dmamem zb_frbufs[ZR_FRAMEBUF];
	} zr_bus;

	/* devfs data */
	struct {
		dev_t	zrdev;
		dev_t	bktrdev;
		dev_t	tunerdev;
		dev_t	vbidev;
		dev_t	bktrdev_alias;
		dev_t	tunerdev_alias;
		dev_t	vbidev_alias;
	} zr_dev;
};

#define ZR_SUBSYSTEM_CONF	0x2c

#define	ZR_VFEHCR_REG		0x000	/* Video Front End, Horizontal Configuration Register */
struct vfehcr {
	u_int32_t	HEnd		:10;
	u_int32_t	HStart		:10;
	u_int32_t			:10;
	u_int32_t	HSPol		:1;
	u_int32_t			:1;
};

#define	ZR_VFEVCR_REG		0x004	/* Video Front End, Vertical Configuration Register */
struct vfevcr {
	u_int32_t	VEnd		:10;
	u_int32_t	VStart		:10;
	u_int32_t			:10;
	u_int32_t	VSPol		:1;
	u_int32_t			:1;
};

#define	ZR_VFESPFR_REG		0x008	/* Video Front End, Scaler and Pixel Format Register */
struct vfespfr {
	u_int32_t	LittleEndian	:1;
	u_int32_t	Pack24		:1;
	u_int32_t	ErrDif		:1;
	u_int32_t	YUV2RGB		:2;	/*
					 * 0: YUV 4:2:2
					 * 1: RGB 8,8,8
					 * 2: RGB 5,6,5 (default)
					 * 3: RGB 5,5,5
					 */
	u_int32_t			:1;
	u_int32_t	DispMod		:1;	/* Display Mode */
	u_int32_t			:1;
	u_int32_t	VerDcm		:6;	/* Vertical Decimination Ratio */
	u_int32_t	HorDcm		:6;	/* Horizontal Decimination Ratio */
	u_int32_t	DupFld		:1;	/* Duplicate Field */
	u_int32_t	HFilter		:3;	/* Horizontal Filter Selection */
	u_int32_t	VCLKPol		:1;	/* Video Clock Polarity */
	u_int32_t	TopField	:1;	/* Top Field Interpretation */
	u_int32_t	ExtFl		:1;	/* External Field Indication */
	u_int32_t			:5;	/* reserved, return zero */
};

#define	ZR_VDTR_REG		0x00c	/* Video Display "Top" Register */
#define	ZR_VDBR_REG		0x010	/* Video Display "Bottom" Register */
#define	ZR_VSSFGR_REG		0x014	/* Video Stride, Status, and Frame Grab Register */
struct vssfgr {
	u_int32_t	FrameGrab	:1;
	u_int32_t	SnapShot	:1;
	u_int32_t			:6;
	u_int32_t	VidOvf		:1;
	u_int32_t			:7;
	u_int32_t	DispStride	:16;
};
#define ZR_VDCR_REG		0x018
struct vdcr {
	u_int32_t	VidWinWid	:10;	/* Video Window Width */
	u_int32_t			:2;	/* reserved */
	u_int32_t	VidWinHt	:10;	/* Video Window Height */
	u_int32_t			:2;	/* reserved */
	u_int32_t	Triton		:1;	/* PCI Bridge Controller Type
					 * 0 - Triton, 1 - other
					 */
	u_int32_t	MinPix		:6;	/* Minimum Number of doublewords (shares with Triton bit) */
	u_int32_t	VidEn		:1;	/* Video Display Enable */
};

#define	ZR_MMTR_REG		0x01c
#define ZR_MMBR_REG		0x020
#define ZR_OCR_REG		0x024
struct ocr {
	u_int32_t	MaskStride	: 8;	/* in doublewords (4 bytes) */
	u_int32_t			: 7;
	u_int32_t	OvlEnable	: 1;
};
	
#define ZR_SYSPCIGPIO_REG	0x028
#define 	ZR_SOFTRESET	(1 << 24)
#define 	ZR_WAITSTATE_SH	16
#define 	ZR_WAITSTATE_MS	(0x7 << ZR_WAITSTATE_SH)
#define 	ZR_GPIO7_DIR	0x80
#define 	ZR_GPIO6_DIR	0x40
#define 	ZR_GPIO5_DIR	0x20
#define 	ZR_GPIO4_DIR	0x10
#define 	ZR_GPIO3_DIR	0x08
#define 	ZR_GPIO2_DIR	0x04
#define 	ZR_GPIO1_DIR	0x02
#define 	ZR_GPIO0_DIR	0x01

#define ZR_GPPR_REG		0x02c
struct gppr {
	u_int32_t	rtg0	: 2;
	u_int32_t	dtg0	: 2;
	u_int32_t	rtg1	: 2;
	u_int32_t	dtg1	: 2;
	u_int32_t	rtg2	: 2;
	u_int32_t	dtg2	: 2;
	u_int32_t	rtg3	: 2;
	u_int32_t	dtg3	: 2;
	u_int32_t		: 8;
	u_int32_t	GenPurIO	: 8;
};

#define ZR_ISR_REG		0x03c
struct isr {
	u_int32_t			:27;
	u_int32_t	JPEGRepIRQ	:1;
	u_int32_t	CodRepIRQ	:1;
	u_int32_t	GIRQ0		:1;
	u_int32_t	GIRQ1		:1;
};

#define ZR_ICR_REG		0x040
struct icr {
	u_int32_t			:24;
	u_int32_t	IntPinEn	:1;
	u_int32_t			:2;
	u_int32_t	JPEGRepIRQEn	:1;
	u_int32_t	CodRepIrqEn	:1;
	u_int32_t	GIRQ0En		:1;
	u_int32_t	GIRQ1En		:1;
};

#define ZR_I2C_REG		0x044
struct i2c
{
	u_int32_t	SCL		:1;
#define ZR_I2C_SCL	0x1
	u_int32_t	SDA		:1;
#define ZR_I2C_SDA	0x2
};

#define ZR_POR_REG		0x200
struct por {
	/* lsb to msb */
	u_int32_t	POData		:8;
	u_int32_t			:8;
	u_int32_t	POGuestReg	:3;
	u_int32_t			:1;
	u_int32_t	POGuestID	:3;
	u_int32_t	PODir		:1;
	u_int32_t	POTime		:1;
	u_int32_t	POPen		:1;
	u_int32_t			:6;
};

#define ZR_GBCR2_REG		0x12c

typedef union {
#define _U_(name)	struct name	name
	u_int32_t	reg;
	_U_(vfehcr);
	_U_(vfevcr);
	_U_(vfespfr);
	_U_(vssfgr);
	_U_(vdcr);
	_U_(ocr);
	_U_(gppr);
	_U_(isr);
	_U_(icr);
	_U_(i2c);
	_U_(por);
#undef _U_
} asrs_t;

/* XXX - protect via lock? */
#define ZR_SET_ASR(zr, regv, mem, bit, val)				\
	do {								\
		asrs_t asr;						\
		asr.reg = zr_read_asrs((zr), (regv));			\
		asr.mem.bit = (val);					\
		zr_write_asrs((zr), (regv), asr.reg);			\
	} while (0)
#define ZR_READ_ASR(zr, regv, mem, bit)	(((asrs_t) {zr_read_asrs((zr), (regv)) }).mem.bit)

struct stat_com_1 {
	u_int32_t	stat_bit	: 1;
	u_int32_t	f_length	: 22;
	u_int32_t			: 1;
	u_int32_t	f_cnt		: 8;
};

typedef union {
	struct stat_com_1	sc_1;
	uint32_t	frag_tab_ptr;	/* must be doubleword aligned */
} stat_com_t;

struct frag_tab_ent {
	u_int32_t	phys_addr;
	u_int32_t	final	: 1;
	u_int32_t	len	: 20;	/* must be multiple of 4 */
	u_int32_t		: 11;
};

#define MIN(a, b)	((a) > (b) ? (b) : (a))
#define ZR_MAXFRAG_ENTS	(MIN((PAGE_SIZE - sizeof(stat_com_t) * ZR_FRAMEBUF) \
			    / sizeof(struct frag_tab_ent) / ZR_FRAMEBUF, \
			    MAXPHYS / PAGE_SIZE))

#endif /* _ZR_REG_H_ */
